SystemOrganization addCategory: #'Magritte-XMLBinding'!
SystemOrganization addCategory: #'Magritte-XMLBinding-Test'!

!MAElementDescription class methodsFor: '*magritte-xmlbinding' stamp: 'pmm 12/6/2006 00:18'!
descriptionXmlAttributeName
	^(MAStringDescription selector: #xmlAttributeName label: 'XML Attribute Name' priority: 1100)! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 15:52'!
beXmlAttribute
	"Store objects as an attribute of the parent node."
	"example: <container attributeName='stringValue' />"
	self 
		beXmlExportable;
		xmlTarget: (MXXmlAttributeTarget new attributeName: self defaultXmlAttributeName)! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/18/2010 21:22'!
beXmlChildAttribute
	"Store objects as an attribute of a child element."
	"example: <container><elementName attributeName='stringValue' /></container>"
	self 
		beXmlExportable;
		xmlTarget: (MXXmlNestedAttributeTarget new 
			elementName: self defaultXmlElementName;
			attributeName: self defaultXmlAttributeName)! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/22/2010 11:01'!
beXmlElement
	"Store objects as the contents of a child element."
	"example: <container><child>....</child></container>"
	
	self 
		beXmlExportable;
		xmlTarget: (MXXmlNamedElementTarget new elementName: self defaultXmlElementName)! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/22/2010 11:18'!
beXmlElementCData
	"Store objects as the contents of a child element. If the contents is Text than a CDATA
	section will be used."
	"example: <container><child><!![CDATA[ ... ]]></child></container>"
	
	self beXmlElement.
	self xmlTarget setUseCDATASection! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 9/16/2010 16:09'!
defaultXmlAttributeName
	"Answer a default xml attribute name. Use the readSelector if present.
	Otherwise convert the label to a valid attribute name."
	
	^self defaultXmlElementName! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 15:54'!
xmlAttributeName
	^self xmlTarget attributeName! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 16:05'!
xmlAttributeName: aString

	self beXmlExportable.
	self propertyAt: #xmlTarget ifAbsent: [
		self xmlTarget: (MXXmlAttributeTarget new attributeName: aString).
		^self ].
	self xmlTarget attributeName: aString! !

!MAReferenceDescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 1/25/2011 12:37'!
beInlined
	self xmlTarget beInlined! !

MAVisitor subclass: #MXXmlBinding
	instanceVariableNames: 'object node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlBinding commentStamp: 'jvds 10/18/2010 21:33' prior: 0!
MXXmlBinding is an abstract superclass for reading and writing xml.

Instance Variables:
	object	<Object>
	node	<XMLElement | nil>!

!MXXmlBinding methodsFor: 'visiting-description' stamp: 'NorbertHartl 2/17/2011 19:19'!
descriptionFor: aContainerObject
	"Answer the description to use for ==aContainerObject==. Subclasses may override this to work with
	alternative descriptions."

	^aContainerObject description! !

!MXXmlBinding methodsFor: 'accessing' stamp: 'jvds 10/7/2010 17:15'!
node
	^ node! !

!MXXmlBinding methodsFor: 'accessing' stamp: 'jvds 10/7/2010 17:15'!
node: anObject
	node := anObject! !

!MXXmlBinding methodsFor: 'accessing' stamp: 'jvds 10/7/2010 17:15'!
object
	^ object! !

!MXXmlBinding methodsFor: 'accessing' stamp: 'jvds 10/7/2010 17:15'!
object: anObject
	object := anObject! !

MXXmlBinding subclass: #MXXmlReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlReader commentStamp: 'jvds 10/18/2010 21:34' prior: 0!
MXXmlReader reads data from XMLElements and creates objects using the Magritte descriptions.!

!MXXmlReader class methodsFor: 'as yet unclassified' stamp: 'jvds 10/6/2010 21:39'!
read: aNode description: aDescription

	^self new read: aNode description: aDescription! !

!MXXmlReader methodsFor: 'public' stamp: 'jvds 10/6/2010 21:49'!
read: aNode description: aDescription

	self 
		node: aNode;
		visit: aDescription.
	^self object! !

!MXXmlReader methodsFor: 'private' stamp: 'jvds 10/6/2010 22:09'!
readText: aDescription

	^aDescription xmlTarget getTextFrom: self node! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'NorbertHartl 12/21/2010 16:11'!
visitContainer: aDescription
	"Create a new instance using the current node and ==aDescription== and read all the children.
	Note that we cannot use ==aDescription== for the children because the object might be an instance
	of a subclass and this subclass may have another description."
	
	| trueDescription |
	
	self object: (aDescription xmlTarget getInstanceFrom: self node description: aDescription).
	
	trueDescription := self descriptionFor: self object.
	(((trueDescription = aDescription)
		ifTrue: [ aDescription ]
		ifFalse: [ trueDescription ])
			children select: [ :each | each isXmlExportable ]) do: [ :each | | value |
				value := each xmlReader read: self node description: each.
				self object write: value using: each ]! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'jvds 10/7/2010 21:44'!
visitElementDescription: aDescription

	| text |
	
	text := self readText: aDescription.
	self object: (text isNil
		ifTrue: [ nil ]
		ifFalse: [ aDescription fromString: text ])
	! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'jvds 10/18/2010 21:12'!
visitSingleOptionDescription: aDescription
	"If the option value is a class than just output the class name instead of the label"
	
	aDescription reference kind = Class
		ifTrue: [
			| text |
			text := self readText: aDescription.
			self object: (text isNil 
				ifTrue: [ nil ] 
				ifFalse: [ Smalltalk environment at: text asSymbol ])]
		ifFalse: [ 
			super visitSingleOptionDescription: aDescription ]! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'NorbertHartl 1/25/2011 15:51'!
visitToManyRelationDescription: aDescription

	| containerNode |
	
	containerNode := aDescription xmlTarget getNodeFrom: self node.
	containerNode isNil
		ifTrue: [ ^self ].

	self object: (OrderedCollection  streamContents: [ :stream |
		containerNode elementsDo: [ :element |
			| value |
			value := aDescription reference xmlReader read: element description: aDescription reference.
			value notNil
				ifTrue: [ stream nextPut: value ]]])
! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'NorbertHartl 11/13/2010 17:06'!
visitToOneRelationDescription: aDescription

	| containerNode |
	
	containerNode := aDescription xmlTarget getNodeFrom: self node.
	containerNode notNil
		ifTrue: [ | value |
			(aDescription reference notNil and: [aDescription reference xmlTarget isKindOf: MXXmlNamedElementTarget]) 
				ifTrue: [ | elem |
					elem := (aDescription reference xmlTarget getNodeFrom: containerNode ).
					value := aDescription reference xmlReader read: elem description: aDescription reference ]
				ifFalse: [ 
					containerNode elementsDo: [ :elem |
						value := aDescription reference xmlReader read: elem description: aDescription reference ]].
			value notNil
				ifTrue: [
					self object: value.
					^ self ]].
	self object: nil! !

MXXmlBinding subclass: #MXXmlWriter
	instanceVariableNames: 'containerNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlWriter commentStamp: 'jvds 10/18/2010 21:34' prior: 0!
MXXmlWriter writes data to XMLElements using an object with its Magritte description.!

!MXXmlWriter class methodsFor: 'as yet unclassified' stamp: 'jvds 10/3/2010 12:18'!
write: anObject description: aDescription to: aNode

	^self new write: anObject description: aDescription to: aNode
! !

!MXXmlWriter methodsFor: 'accessing' stamp: 'jvds 10/5/2010 22:10'!
containerNode
	^ containerNode! !

!MXXmlWriter methodsFor: 'accessing' stamp: 'jvds 10/5/2010 22:10'!
containerNode: anObject
	containerNode := anObject! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'jvds 10/10/2010 21:50'!
visitContainer: aDescription
	"Write out all the elements of object that are xml exportable
	and are notNil."
	
	self node: (aDescription xmlTarget addNodeForInstance: self object to: self containerNode).
	
	(aDescription children select: [ :each | each isXmlExportable ]) do: [ :each |
		each xmlWriter 
			write: self object description: each to: self node ]! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'NorbertHartl 11/22/2010 21:16'!
visitElementDescription: aDescription

	| value text |

	value := self object readUsing: aDescription.
	text := value ifNotNilDo: [ :v | aDescription toString: v ].
	self writeText: text description: aDescription! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'NorbertHartl 11/30/2010 12:14'!
visitSingleOptionDescription: aDescription
	"If the option value is a class than just output the class name instead of the label"
	
	aDescription reference kind = Class
		ifTrue: [
			self 
				writeText: ((self object readUsing: aDescription) ifNotNilDo: [ :c | c name greaseString ])
				description: aDescription ]
		ifFalse: [ 
			super visitSingleOptionDescription: aDescription ]! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'NorbertHartl 2/17/2011 19:21'!
visitToManyRelationDescription: aDescription

	| collection referenceDescription |
	
	collection := self object readUsing: aDescription.
	collection isNil
		ifFalse: [
			self node: (aDescription xmlTarget addNodeTo: self containerNode).
			collection do: [ :each |
				referenceDescription := aDescription reference ifNil: [ self descriptionFor: each ].
				referenceDescription xmlWriter
					write: each description: referenceDescription to: self node ]].! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'NorbertHartl 2/17/2011 19:21'!
visitToOneRelationDescription: aDescription

	| value referenceDescription |
	value := self object readUsing: aDescription.
	value isNil
		ifTrue: [
			self node: (aDescription xmlTarget addUndefinedTo: self containerNode)]
		ifFalse: [
			self node: (aDescription xmlTarget addNodeTo: self containerNode).
			referenceDescription := aDescription reference ifNil: [ self descriptionFor: value].
			referenceDescription xmlWriter
				write: value description: referenceDescription to: self node ]! !

!MXXmlWriter methodsFor: 'public' stamp: 'jvds 10/4/2010 21:31'!
write: anObject description: aDescription to: aNode

	self 
		containerNode: aNode;
		object: anObject;
		visit: aDescription.
	^self node! !

!MXXmlWriter methodsFor: 'private' stamp: 'jvds 10/18/2010 21:05'!
writeText: aString description: aDescription

	self node: (
		aString isNil
			ifTrue: [ aDescription xmlTarget addUndefinedTo: self containerNode ]
			ifFalse: [ aDescription xmlTarget addText: aString to: self containerNode ])! !

!MAContainer methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/5/2010 23:28'!
defaultXmlTarget
	"Answer a default xml element name. Use the readSelector if present.
	Otherwise convert the label to a valid element name."

	^MXXmlClassNameElementTarget new! !

!MAContainer methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/11/2010 18:26'!
xmlElementName: anElementName classAttribute: anAttributeName

	self beXmlExportable.
	self xmlTarget: (MXXmlClassNameAttributeTarget new
		elementName: anElementName;
		classAttributeName: anAttributeName)! !

!MARelationDescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 11/13/2010 17:30'!
beXmlInlineElement
	"Store related objects directly in the container node."
	
	self 
		beXmlExportable;
		xmlTarget: (MXXmlElementTarget new) beInlined  ! !

MADescriptionBuilder subclass: #MXTestDescriptionBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXTestDescriptionBuilder methodsFor: 'as yet unclassified' stamp: 'jvds 10/10/2010 15:18'!
add: aDescription for: anObject

	cache at: anObject put: aDescription! !

!MXTestDescriptionBuilder methodsFor: 'as yet unclassified' stamp: 'jvds 10/10/2010 15:09'!
build: anObject
	self error: 'User add:for: to initialize the descriptions.'! !

TestCase subclass: #MXXmlObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXXmlObjectTest methodsFor: 'fixtures' stamp: 'NorbertHartl 2/21/2010 08:04'!
simpleObject
	^ MXSimpleObject new
		title: 'A title';
		abstract: 'A longer text as abstract';
		token: 'article';
		index: 100000 atRandom;
		date: Date today! !

!MXXmlObjectTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 13:14'!
testAttributeWrite
	| object xml |
	object := MXSimpleObject new
		title: 'A title'.
	xml := object toXml.
	self assert: (xml attributeAt: 'title') = 'A title'! !

!MXXmlObjectTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 13:16'!
testChildNodeAttributeWrite
	| object xml |
	object := MXSimpleObject new
		token: 'a subelement attribute'.
	xml := object toXml.
	"<simpleobject><token type='a subelement attribute'/> </simpleobject>"
	self assert: (xml elementAt: 'token') notNil.
	self assert: ((xml elementAt: 'token') attributeAt: 'type') = 'a subelement attribute'! !

!MXXmlObjectTest methodsFor: 'testing' stamp: 'NorbertHartl 3/16/2010 14:12'!
testChildNodeContentWrite
	| object xmlString |
	object := MXSimpleObject new
		abstract: 'Some words as abstract'.
	xmlString := String streamContents: [:stream|
		object printXMLOn: stream].
	self assert: xmlString = '<simpleobject><abstract>Some words as abstract</abstract></simpleobject>'! !

!MXXmlObjectTest methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 12/21/2010 16:08'!
testDescriptionConfigurationPropagation
	| object desc xml result |
	object := self simpleObject.
	desc := object description collect: [:each| 
		(each label = 'Abstract') 
			ifTrue: [
				each copy
					xmlReadBlock: [:elem| elem attributeAt: 'customAbstract'] 
					writeBlock: [:elem :obj| elem attributeAt: 'customAbstract' put: obj ]]
			ifFalse: [ each ]].
	
	xml := desc toXml: object.
	result := desc fromXml: xml.
	self assert: result abstract = object abstract
	! !

!MXXmlObjectTest methodsFor: 'testing' stamp: 'NorbertHartl 3/16/2010 14:12'!
testNodeContentWrite
	| object xmlString |
	object := MXSimpleObject new
		abstract: 'Some words as abstract'.
	xmlString := String streamContents: [:stream|
		object printXMLOn: stream].
	self assert: xmlString = '<simpleobject><abstract>Some words as abstract</abstract></simpleobject>'! !

!MXXmlObjectTest methodsFor: 'testing' stamp: 'jvds 10/18/2010 22:19'!
testObjectRoundtrip
	| object xmlString created |
	object := self simpleObject.
	xmlString := String streamContents: [:stream|
		object printXMLOn: stream].
	created := MXSimpleObject fromXml: xmlString.

	self assert: (object title = created title).
	self assert: (object index = created index).
	self assert: (object date = created date).
	self assert: (object abstract = created abstract).
	self assert: (object token = created token)! !

!MXXmlObjectTest methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 12/21/2010 15:19'!
testXmlDocumentCreationFromDescription
	| object |
	object := self simpleObject.
	
	self assert: ((object description toXmlNode: object) class = XMLElement).
	self assert: ((object description toXmlDocument: object) class = XMLDocument ).
! !

!MXXmlObjectTest methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 12/21/2010 15:17'!
testXmlDocumentCreationFromObject
	| object |
	object := self simpleObject.
	
	self assert: (object toXml class = XMLElement).
	self assert: ((object toXmlDocument) class = XMLDocument ).
! !

TestCase subclass: #MXXmlPersonReferenceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXXmlPersonReferenceTest methodsFor: 'as yet unclassified' stamp: 'jvds 10/9/2010 23:54'!
testMultipleOption

	| person xml created |
	
	person := MXTestPerson new
		name: 'Pete';
		otherHobbies: #( 'Tennis' 'Golf' );
		yourself.
		
	xml := person description toXmlNode: person.
	self assert: (xml asString includesSubString: '<otherHobbies>Tennis, Golf</otherHobbies>').
	
	created := MXTestPerson description fromXmlNode: xml.
	self assert: (person otherHobbies = created otherHobbies).! !

!MXXmlPersonReferenceTest methodsFor: 'as yet unclassified' stamp: 'jvds 10/9/2010 23:54'!
testSingleOption

	| person xml created |
	
	person := MXTestPerson new
		name: 'Pete';
		favouriteHobby: 'Tennis';
		yourself.
		
	xml := person description toXmlNode: person.
	
	created := MXTestPerson description fromXmlNode: xml.
	
	self assert: (person favouriteHobby = created favouriteHobby).! !

!MXXmlPersonReferenceTest methodsFor: 'as yet unclassified' stamp: 'jvds 10/22/2010 09:21'!
testToManyRelation

	| person xml created |
	
	person := MXTestPerson new
		name: 'Pete';
		otherAddresses: (OrderedCollection new
			add: (MXTestStreetAddress new
				street: 'Noordeinde';
				city: 'Delft';
				postalCode: '2611KH');
			add: (MXTestStreetAddress new
				street: 'Rubenslaan';
				city: 'Utrecht';
				postalCode: '3582JD');
			yourself).
		
	xml := person description toXmlNode: person.
	
	created := MXTestPerson description fromXmlNode: xml.
	
	self assert: (person otherAddresses size = created otherAddresses size).
	self assert: (person otherAddresses last street = created otherAddresses last street).! !

!MXXmlPersonReferenceTest methodsFor: 'as yet unclassified' stamp: 'jvds 10/22/2010 09:21'!
testToOneRelation

	| person xml created |
	
	person := MXTestPerson new
		name: 'Pete';
		address: (MXTestStreetAddress new
			street: 'Noordeinde';
			city: 'Delft';
			postalCode: '2611KH';
			yourself);
		yourself.
		
	xml := person description toXmlNode: person.
	
	created := MXTestPerson description fromXmlNode: xml.
	
	self assert: (person name = created name).
	self assert: (person address class = created address class).
	self assert: (person address postalCode = created address postalCode)! !

TestCase subclass: #MXXmlPersonTargetTest
	instanceVariableNames: 'oldBuilder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXXmlPersonTargetTest methodsFor: 'running' stamp: 'jvds 10/10/2010 15:16'!
register: aDescription for: anObject

	MADescriptionBuilder default add: aDescription for: anObject! !

!MXXmlPersonTargetTest methodsFor: 'running' stamp: 'jvds 10/10/2010 15:12'!
setUp

	super setUp.
	
	oldBuilder := MADescriptionBuilder default.
	MADescriptionBuilder default: MXTestDescriptionBuilder new! !

!MXXmlPersonTargetTest methodsFor: 'running' stamp: 'jvds 10/10/2010 15:12'!
tearDown

	oldBuilder notNil
		ifTrue: [ MADescriptionBuilder default: oldBuilder ].
	super tearDown! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/10/2010 16:05'!
testAttributeCustom

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		add: (MAStringDescription new
			accessor: #name;
			label: 'Name';
			xmlAttributeName: 'p-name' );
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<MAPerson p-name='Pepe'></MAPerson>"
	self assert: (node attributeAt: 'p-name') = person name.
	
	created := description fromXmlNode: node.
	self assert: created name = person name.! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/10/2010 15:19'!
testAttributeDefault

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		add: (MAStringDescription new
			accessor: #name;
			label: 'Name';
			beXmlAttribute );
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<MAPerson name='Pepe'></MAPerson>"
	self assert: (node attributeAt: 'name') = person name.
	
	created := description fromXmlNode: node.
	self assert: created name = person name.! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/10/2010 22:40'!
testAttributeNested

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		add: (MAStringDescription new
			accessor: #name;
			label: 'Name';
			beXmlChildAttribute );
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<MXTestPerson><name name='Pepe'/></MXTestPerson>"
	self assert: ((node elementAt: 'name') attributeAt: 'name') = person name.
	
	created := description fromXmlNode: node.
	self assert: created name = person name.! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/10/2010 22:41'!
testAttributeNestedCustom

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		add: (MAStringDescription new
			accessor: #name;
			label: 'Name';
			beXmlChildAttribute;
			xmlElementName: 'a';
			xmlAttributeName: 'b' );
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<MXTestPerson><a b='Pepe'/></MXTestPerson>"
	self assert: ((node elementAt: 'a') attributeAt: 'b') = person name.
	
	created := description fromXmlNode: node.
	self assert: created name = person name.! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/10/2010 16:06'!
testBlock

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		add: (MAStringDescription new
			accessor: #name;
			label: 'Name';
			xmlReadBlock: [ :elem | elem attributeAt: 'blck' ]
				writeBlock: [ :elem :object | elem attributeAt: 'blck' put: object ]
			yourself );
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<MAPerson blck='Pepe'></MAPerson>"
	self assert: (node attributeAt: 'blck') = person name.
	
	created := description fromXmlNode: node.
	self assert: created name = person name.! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 13:13'!
testBlockWithNil

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		add: (MAStringDescription new
			accessor: #name;
			label: 'Name';
			xmlReadBlock: [ :elem | elem attributeAt: 'blck' ifAbsent: [ nil ]]
				writeBlock: [ :elem :object | object notNil ifTrue: [ elem attributeAt: 'blck' put: object ]]
			yourself );
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: nil.
		
	node := description toXmlNode: person.
	
	"<MAPerson></MAPerson>"
	self assert: (node attributeAt: 'blck' ifAbsent: [ nil ]) isNil.
	
	created := description fromXmlNode: node.
	self assert: created name isNil.! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/10/2010 16:16'!
testContainerElementCustom

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		kind: MXTestPerson;
		xmlElementName: 'person';
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<person></person>"
	self assert: node name = 'person'.
	
	created := description fromXmlNode: node.
	self assert: created class = person class! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/11/2010 18:28'!
testContainerElementWithTypeAttribute

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		kind: MXTestPerson;
		xmlElementName: 'person' classAttribute: 'stclass';
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<person stclass='MXTestPerson'></person>"
	self assert: node name = 'person'.
	self assert: (node attributeAt: 'stclass') = 'MXTestPerson'.
	
	created := description fromXmlNode: node.
	self assert: created class = person class! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/10/2010 16:02'!
testElementCustom

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		add: (MAStringDescription new
			accessor: #name;
			label: 'Name';
			xmlElementName: 'p-name' );
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<MAPerson><p-name>Pepe</p-name></MAPerson>"
	self assert: (node elementAt: 'p-name') contentString = person name.
	
	created := description fromXmlNode: node.
	self assert: created name = person name.! !

!MXXmlPersonTargetTest methodsFor: 'testing' stamp: 'jvds 10/10/2010 15:17'!
testElementDefault

	| description person node created |
	
	description := MXTestPerson descriptionContainer
		add: (MAStringDescription new
			accessor: #name;
			label: 'Name';
			beXmlElement );
		yourself.
	self register: description for: MXTestPerson.
		
	person := MXTestPerson new
		name: 'Pepe'.
		
	node := description toXmlNode: person.
	
	"<MAPerson><name>Pepe</name></MAPerson>"
	self assert: (node elementAt: 'name') contentString = person name.
	
	created := description fromXmlNode: node.
	self assert: created name = person name.! !

TestCase subclass: #MXXmlRelationEmbedTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXXmlRelationEmbedTest methodsFor: 'fixtures' stamp: 'NorbertHartl 3/15/2010 18:59'!
embedObject
	^ MXRelationEmbedSource new
		title: 'A title';
		target: (
			MXRelationEmbedTarget new
				name: 'A name')! !

!MXXmlRelationEmbedTest methodsFor: 'testing' stamp: 'jvds 8/19/2010 16:43'!
testEmbedWrite
	| object xmlString |
	object := self embedObject.
	xmlString := String streamContents: [:stream|
		object printXMLOn: stream].
	self assert: (xmlString indexOfSubCollection: 'embedTargetName="A name"') > 0.
	self assert: (xmlString indexOfSubCollection: 'title="A title"') > 0! !

TestCase subclass: #MXXmlRelationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXXmlRelationTest methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 2/17/2010 09:04'!
findTitle: aString in: aCollection
	^ (aCollection 
		detect: [:each| each title = aString]
		ifNone: [ nil ]) notNil! !

!MXXmlRelationTest methodsFor: 'fixtures' stamp: 'NorbertHartl 2/17/2010 13:37'!
onoToMany
	^ MXOneToManySource new
		label: 'A label';
		targets: (
			OrderedCollection new
				add:(MXRelationTarget new
					title: 'title1');
				add:(MXRelationTarget new
					title: 'title2');
				add:(MXRelationTarget new
					title: 'title3');
				yourself
			);
		inlineTargets: (
			OrderedCollection new
				add:(MXRelationInlineTarget new
					title: 'inline title1');
				add:(MXRelationInlineTarget new
					title: 'inline title2');
				add:(MXRelationInlineTarget new
					title: 'inline title3');
				yourself
			)! !

!MXXmlRelationTest methodsFor: 'fixtures' stamp: 'NorbertHartl 2/17/2010 13:37'!
onoToOne
	^ MXOneToOneSource new
		label: 'A label';
		target: (
			MXRelationTarget new
				title: 'A title');
		inlineTarget: (
			MXRelationInlineTarget new
				title: 'An inline title')! !

!MXXmlRelationTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 13:08'!
testOneToManyElementWrite
	| object xml |
	object := MXOneToManySource new
		label: 'A label';
		targets: (
			OrderedCollection new
				add:(MXRelationTarget new
					title: 'title1');
				add:(MXRelationTarget new
					title: 'title2');
				yourself).
	xml := object toXml.
	"<toManySource label='A label'><elementTargets><elementtarget title='title1'/> <elementtarget title='title2'/></elementTargets></toManySource>"
	self assert: (xml elementAt: 'elementTargets') notNil.
	self assert: ((xml elementAt: 'elementTargets') elementsAt: 'elementtarget') size = 2! !

!MXXmlRelationTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 13:06'!
testOneToManyInlineWrite
	| object xml |
	object := MXOneToManySource new
		label: 'A label';
		inlineTargets: (
			OrderedCollection new
				add:(MXRelationInlineTarget new
					title: 'title1');
				add:(MXRelationInlineTarget new
					title: 'title2');
				yourself).
	xml := object toXml.
	"<toManySource label='A label'><inlineTarget title='title1'/> <inlineTarget title='title2'/> </toManySource>'"
	self assert: (xml elementsAt: 'inlineTarget') size = 2
! !

!MXXmlRelationTest methodsFor: 'testing' stamp: 'NorbertHartl 3/10/2010 16:15'!
testOneToManyRoundtrip
	| object xmlString created |
	object := self onoToMany.
	xmlString := String streamContents: [:stream|
		object printXMLOn: stream].
	created := MXOneToManySource fromXml: xmlString.
	self assert: object label = created label.
	self assert: created targets notNil.
	self assert: created targets notEmpty.
	self assert: object targets size = created targets size.
	self assert: (self findTitle: 'title1' in: created targets).
	self assert: (self findTitle: 'title2' in: created targets).
	self assert: (self findTitle: 'title3' in: created targets).
	self assert: created inlineTargets notEmpty.
	self assert: (self findTitle: 'inline title1' in: created inlineTargets).
	self assert: (self findTitle: 'inline title2' in: created inlineTargets).
	self assert: (self findTitle: 'inline title3' in: created inlineTargets).
! !

!MXXmlRelationTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 13:04'!
testOneToOneElementWrite
	| object xml |
	object := MXOneToOneSource new
		label: 'A label';
		target: (
			MXRelationTarget new
				title: 'A title').
	xml := object toXml.
	"<toOneSource label='A label'><toOne><elementtarget title='A title'/> </toOne></toOneSource>"
	self assert: (xml elementAt: 'toOne') notNil.
	self assert: ((xml elementAt: 'toOne') elementAt: 'elementtarget') notNil! !

!MXXmlRelationTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 13:00'!
testOneToOneInlineWrite
	| object xml |
	object := MXOneToOneSource new
		label: 'A label';
		inlineTarget: (
			MXRelationInlineTarget new
				title: 'A title').
	xml := object toXml.
	
	"<toOneSource label='A label'><inlineTarget title='A title'/> </toOneSource>"
	self assert: (xml elementAt: 'inlineTarget') notNil.
	self assert: (xml elementAt: 'inlineTarget') hasElements not.
	self assert: ((xml elementAt: 'inlineTarget') attributeAt: 'title') = 'A title'
	! !

!MXXmlRelationTest methodsFor: 'testing' stamp: 'jvds 8/30/2010 22:20'!
testOneToOneRoundtrip
	| object xmlString created |
	object := self onoToOne.
	xmlString := String streamContents: [:stream|
		object printXMLOn: stream].
	created := MXOneToOneSource fromXml: xmlString.
	self assert: object label = created label.
	self assert: created target notNil.
	self assert: object target title = created target title.
	self assert: object inlineTarget title = created inlineTarget title.
! !

TestCase subclass: #MXXmlTargetTest
	instanceVariableNames: 'element person'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:44'!
doTestText: aXmlTarget

	| value |
	
	aXmlTarget addText: 'text' to: self element.
	value := aXmlTarget getTextFrom: self element.
	self assert: (value = 'text')
! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:52'!
doTestTextEmpty: aXmlTarget

	| value |
	
	value := aXmlTarget getTextFrom: self element.
	self assert: value isNil
! !

!MXXmlTargetTest methodsFor: 'accessing' stamp: 'jvds 10/22/2010 09:33'!
element

	^element ifNil: [ element := XMLElement named: 'test' ]! !

!MXXmlTargetTest methodsFor: 'accessing' stamp: 'jvds 10/22/2010 10:00'!
person

	^person ifNil: [
		person := MXTestPerson new
			name: 'Pete']! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'NorbertHartl 11/13/2010 17:28'!
testNodeInlineElement

	| target node resultNode |
	
	target := MXXmlElementTarget new beInlined.
		
	node := target addNodeTo: self element.
	self assert: (node isKindOf: XMLElement).
	self assert: node = self element.
	
	resultNode := target getNodeFrom: self element.
	self assert: node = resultNode.
	! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:50'!
testNodeNamedElement

	| target node resultNode |
	
	target := MXXmlNamedElementTarget new
		elementName: 'pipo'.
		
	node := target addNodeTo: self element.
	self assert: (node isKindOf: XMLElement).
	self assert: node name = 'pipo'.
	
	resultNode := target getNodeFrom: self element.
	self assert: node = resultNode.
	! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:51'!
testNodeNamedElementEmpty

	| target resultNode |
	
	target := MXXmlNamedElementTarget new
		elementName: 'pipo'.
		
	resultNode := target getNodeFrom: self element.
	self assert: resultNode isNil.
	! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:37'!
testTextAttribute

	| target |
	
	target := MXXmlAttributeTarget new
		attributeName: 'pipo'.
		
	self doTestText: target.
	self assert: (self element attributeAt: 'pipo') = 'text'.! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:53'!
testTextAttributeEmpty

	| target |
	
	target := MXXmlAttributeTarget new
		attributeName: 'pipo'.
		
	self doTestTextEmpty: target! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:40'!
testTextNamedElement

	| target |
	
	target := MXXmlNamedElementTarget new
		elementName: 'pipo'.
		
	self doTestText: target.
	self assert: (self element elementAt: 'pipo') notNil.
	self assert: (self element elementAt: 'pipo') contentString = 'text'.! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 13:04'!
testTextNamedElementCData

	| target |
	
	target := MXXmlNamedElementTarget new
		elementName: 'pipo';
		setUseCDATASection.
		
	self doTestText: target.
	self assert: (self element elementAt: 'pipo') contentString = 'text'.
	self assert: ((self element elementAt: 'pipo') firstNode isKindOf: XMLCData)! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:53'!
testTextNamedElementEmpty

	| target |
	
	target := MXXmlNamedElementTarget new
		elementName: 'pipo'.
		
	self doTestTextEmpty: target! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:42'!
testTextNestedAttribute

	| target |
	
	target := MXXmlNestedAttributeTarget new
		elementName: 'e-pipo';
		attributeName: 'pipo'.
		
	self doTestText: target.
	self assert: (self element elementAt: 'e-pipo') notNil.
	self assert: ((self element elementAt: 'e-pipo') attributeAt: 'pipo') = 'text'.! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 09:53'!
testTextNestedAttributeEmpty

	| target |
	
	target := MXXmlNestedAttributeTarget new
		elementName: 'e-pipo';
		attributeName: 'pipo'.
		
	self doTestTextEmpty: target! !

!MXXmlTargetTest methodsFor: 'testing' stamp: 'jvds 10/22/2010 13:00'!
testUndefined

	| target |
	
	target := MXXmlAttributeTarget new
		attributeName: 'pipo'.
		
	target addUndefinedTo: self element.
	self assert: (self element elementAt: 'pipo') isNil! !

TestCase subclass: #MXXmlTypesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 8/5/2010 00:12'!
testBooleanRead
	| created xmlString |
	xmlString := '<typesobject boolean="true"/> '.
	created := MXTypesObject fromXml: xmlString.
	
	self assert: created boolean = true! !

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 12:44'!
testBooleanWrite
	| object xml |
	object := MXTypesObject new
		boolean: true.
	xml := object toXml.
	"<typesobject boolean='true'/>"
	self assert: (xml attributeAt: 'boolean') = 'true'! !

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 12:45'!
testDateWrite
	| object xml |
	object := MXTypesObject new
		date: (DateAndTime year: 2010 month: 3 day: 10 hour: 4 minute: 4 second: 4 offset: Duration zero).
	xml := object toXml.
	" '<typesobject date='2010-03-10T04:04:04+00:00'/> "
	self assert: (xml attributeAt: 'date') = '2010-03-10T04:04:04+00:00'! !

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 8/31/2010 21:48'!
testMultipleOptionsRead

	| created xmlString |

	xmlString := '<typesobject><colors>yellow, green</colors></typesobject>'.
	created := MXTypesObject fromXml: xmlString.

	self assert: created colors = #( yellow green )! !

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 8/31/2010 21:46'!
testMultipleOptionsWrite

	| object xmlString |
	object := MXTypesObject new
		colors: #( yellow green ).
		
	xmlString := String streamContents: [ :stream |
		object printXMLOn: stream].
	self assert: xmlString = '<typesobject><colors>yellow, green</colors></typesobject>'! !

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 12:46'!
testNumberWrite
	| object xml |
	object := MXTypesObject new
		number: 12345.
	xml := object toXml.
	self assert: (xml attributeAt: 'number') = '12345'! !

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 8/8/2010 15:48'!
testSingleOptionRead

	| created xmlString |

	xmlString := '<typesobject magnitudeClass="Integer"/> '.
	created := MXTypesObject fromXml: xmlString.
	
	self assert: created magnitudeClass = Integer! !

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 12:47'!
testSingleOptionWrite

	| object xml |
	object := MXTypesObject new
		magnitudeClass: Integer.
	xml := object toXml.
	self assert: (xml attributeAt: 'magnitudeClass') = 'Integer'! !

!MXXmlTypesTest methodsFor: 'testing' stamp: 'jvds 11/6/2010 12:48'!
testStringWrite
	| object xml |
	object := MXTypesObject new
		string: 'a string'.
	xml := object toXml.
	self assert: (xml attributeAt: 'string') = 'a string'! !

!MADescription class methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/18/2010 21:27'!
defaultXmlReader
	^MXXmlReader ! !

!MADescription class methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/18/2010 21:27'!
defaultXmlWriter
	^MXXmlWriter ! !

!MADescription class methodsFor: '*magritte-xmlbinding' stamp: 'pmm 12/5/2006 18:42'!
descriptionXmlElementName
	^(MAStringDescription selector: #xmlElementName label: 'XML Element Name' priority: 1000)! !

!MADescription class methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 02/22/2010 00:26'!
descriptionXmlReader
	^ (MASingleOptionDescription selector: #xmlReader label: 'XML Reader' priority: 1300 default: self defaultXmlReader)
		reference: MAClassDescription new;
		options: (Array
			with: MXXmlReader);
		yourself ! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 2/19/2010 17:53'!
beXmlExportable
	self propertyAt: #xmlExportable put: true! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 9/16/2010 08:54'!
defaultXmlElementName
	"Answer a default xml element name. Use the readSelector if present.
	Otherwise convert the label to a valid element name."

	| name |
	
	(self accessor respondsTo: #readSelector)
		ifTrue: [ ^self accessor readSelector greaseString ].
		
	name := self label select: [ :char | char isAlphaNumeric ].
	name size > 0
		ifTrue: [ name at: 1 put: name first asLowercase ].
	^name! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/5/2010 23:27'!
defaultXmlTarget
	"Answer a default xml element name. Use the readSelector if present.
	Otherwise convert the label to a valid element name."

	^MXXmlNamedElementTarget new elementName: self defaultXmlElementName! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/14/2010 14:24'!
fromXml: aString
	"Answer an object being parsed from <code>aString</code>."

	^ self fromXml: aString reader: self xmlReader! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 8/31/2010 10:30'!
fromXml: aString reader: aParser
	"Answer an object being parsed from <code>aNode</code> using <code>aParser</code>."

	| node |
	node :=  (XMLDOMParser parseDocumentFrom: aString readStream) root.
	^self fromXmlNode: node reader: aParser! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/6/2010 21:40'!
fromXmlNode: aNode 
	"Answer an object being parsed from <code>aNode</code>."

	^self fromXmlNode: aNode reader: self xmlReader! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/6/2010 21:41'!
fromXmlNode: aNode reader: aXmlReader
	"Answer an object being parsed from <code>aNode</code> using <code>aXmlReader</code>."

	^aXmlReader read: aNode description: self! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 2/19/2010 17:53'!
isXmlExportable
	^ self propertyAt: #xmlExportable ifAbsent: [ false ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 7/6/2011 15:35'!
toXml: anObject
	"Answer a String created from anObject using the receiver as the description."
	
	| xml |
	xml := self toXmlNode: anObject.	
	^String streamContents: [ :stream | xml writeXMLOn: (XMLWriter on: stream)]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 12/21/2010 15:02'!
toXmlDocument: anObject
	"Answer a XML Document created from anObject using the receiver as the description."

	^ (self toXmlNode: anObject) document! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 12/21/2010 15:02'!
toXmlNode: anObject
	"Answer a XML Node created from anObject using the receiver as the description."

	^ self toXmlNode: anObject parent: XMLDocument new! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 12/21/2010 15:00'!
toXmlNode: anObject parent: aParentNode
	"Answer a XML Node created from anObject using the receiver as the description. If the receivers
	xmlNode is MXSameNode than anObject will be added to aParentNode. Otherwise aParentNode
	will be ignored."

	^ self xmlWriter write: anObject description: self to: aParentNode.
	! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 16:00'!
xmlElementName
	^self xmlTarget elementName! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 16:03'!
xmlElementName: aString

	self beXmlExportable.
	self propertyAt: #xmlTarget ifAbsent: [
		self xmlTarget: (MXXmlNamedElementTarget new elementName: aString).
		^self ].
	self xmlTarget elementName: aString! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 15:37'!
xmlReadBlock: aReadBlock writeBlock: aWriteBlock

	| readerAndWriter |
	
	readerAndWriter := MXPluggableXmlAccessor new
		readBlock: aReadBlock;
		writeBlock: aWriteBlock;
		yourself.
		
	self 
		beXmlExportable;
		xmlReader: readerAndWriter;
		xmlWriter: readerAndWriter.! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 8/31/2010 11:59'!
xmlReader
	^self propertyAt: #xmlReader ifAbsent: [ self class defaultXmlReader ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 8/7/2010 12:19'!
xmlReader: aReader
	self propertyAt: #xmlReader put: aReader! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/5/2010 22:02'!
xmlTarget

	^self propertyAt: #xmlTarget ifAbsentPut: [ self defaultXmlTarget ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/5/2010 22:02'!
xmlTarget: aXmlTarget

	^self propertyAt: #xmlTarget put: aXmlTarget! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 16:00'!
xmlWriter
	^self propertyAt: #xmlWriter ifAbsent: [ self class defaultXmlWriter ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 2/21/2010 18:15'!
xmlWriter: aWriter
	self propertyAt: #xmlWriter put: aWriter! !

Object subclass: #MXObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXObject commentStamp: 'jvds 10/18/2010 21:30' prior: 0!
MXObject can be used as a superclass for objects that need to be marshalled to xml.!

MXObject subclass: #MXFileObject
	instanceVariableNames: 'file abstract'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXFileObject class methodsFor: 'as yet unclassified' stamp: 'jvds 8/9/2010 15:12'!
descriptionAbstract

	^ MAStringDescription new
		accessor: #abstract;
		label: 'Abstract';
		beXmlExportable;	
		yourself! !

!MXFileObject class methodsFor: 'as yet unclassified' stamp: 'jvds 10/14/2010 14:32'!
descriptionFile

	^ MAFileDescription new
		accessor: #file;
		label: 'File';
		beXmlExportable;	
		yourself! !

!MXFileObject class methodsFor: 'as yet unclassified' stamp: 'jvds 8/9/2010 15:09'!
descriptionFilePharo: aDescription

	^ aDescription
		kind: MAExternalFileModel;
		yourself! !

!MXFileObject class methodsFor: 'accessing' stamp: 'jvds 8/9/2010 15:14'!
fileModel
	^ (self description detect: [ :each |
		each = self descriptionFile ])
			kind! !

!MXFileObject class methodsFor: 'testing' stamp: 'jvds 8/9/2010 15:52'!
isAbstract
	^false! !

!MXFileObject class methodsFor: 'accessing' stamp: 'jvds 8/9/2010 15:38'!
xmlElementName
	^ 'fileobject'! !

!MXFileObject methodsFor: 'accessing' stamp: 'jvds 8/9/2010 15:11'!
abstract
	^ abstract! !

!MXFileObject methodsFor: 'accessing' stamp: 'jvds 8/9/2010 15:11'!
abstract: anObject
	abstract := anObject! !

!MXFileObject methodsFor: 'accessing' stamp: 'jvds 8/9/2010 15:14'!
file
	^ file ifNil: [ file := self class fileModel new ]! !

!MXFileObject methodsFor: 'accessing' stamp: 'jvds 8/9/2010 15:11'!
file: anObject
	file := anObject! !

!MXObject class methodsFor: 'accessing-description' stamp: 'NorbertHartl 02/18/2010 14:41'!
descriptionContainer
	^super descriptionContainer
		xmlElementName: self xmlElementName;
		yourself
! !

!MXObject class methodsFor: 'accessing' stamp: 'NorbertHartl 02/18/2010 09:49'!
forElementNamed: aString
	self withAllConcreteClassesDo: [ :each |
		(each description xmlElementName = aString)
			ifTrue: [ ^each new ] ].
	self error: 'unknown element name: ', aString! !

!MXObject class methodsFor: 'instance creation' stamp: 'NorbertHartl 2/15/2010 11:27'!
fromXml: aString
	^self description fromXml: aString! !

!MXObject class methodsFor: 'instance creation' stamp: 'jvds 10/18/2010 22:17'!
fromXmlNode: aString
	^self description fromXmlNode: aString! !

!MXObject class methodsFor: 'testing' stamp: 'NorbertHartl 2/15/2010 11:27'!
isAbstract
	^ true! !

!MXObject class methodsFor: 'reflection' stamp: 'NorbertHartl 2/15/2010 11:27'!
withAllConcreteClasses
	^Array streamContents: [ :stream |
		self withAllConcreteClassesDo: [ :each |
			stream nextPut: each ] ]! !

!MXObject class methodsFor: 'reflection' stamp: 'NorbertHartl 2/15/2010 11:27'!
withAllConcreteClassesDo: aBlock
	self withAllSubclassesDo: [ :each |
		each isAbstract ifFalse: [
			aBlock value: each ] ]! !

!MXObject class methodsFor: 'accessing' stamp: 'NorbertHartl 02/18/2010 14:42'!
xmlElementName
	self subclassResponsibility! !

!MXObject methodsFor: 'printing' stamp: 'jvds 8/31/2010 13:17'!
printXMLOn: aStream
	self toXml printXMLOn: (XMLWriter on: aStream)! !

!MXObject methodsFor: 'converting' stamp: 'NorbertHartl 12/21/2010 14:40'!
toXml
	^ self toXml: XMLDocument new! !

!MXObject methodsFor: 'converting' stamp: 'jvds 8/31/2010 12:23'!
toXml: aParentNode
	^ self toXml: aParentNode using: self description! !

!MXObject methodsFor: 'converting' stamp: 'jvds 8/31/2010 12:23'!
toXml: aParentNode using: aDescription
	^aDescription toXmlNode: self parent: aParentNode! !

!MXObject methodsFor: 'converting' stamp: 'NorbertHartl 12/21/2010 15:14'!
toXmlDocument
	^ self toXml document! !

MXObject subclass: #MXOneToManySource
	instanceVariableNames: 'label targets inlineTargets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXOneToManySource class methodsFor: 'description' stamp: 'jvds 10/18/2010 18:28'!
descriptionInlineTargets
	^(MAToManyRelationDescription new 
		accessor: #inlineTargets; 
		label: '1:m inline targets'; 
		priority: 600)
		reference: MXRelationInlineTarget description;
		beXmlInlineElement;
		yourself! !

!MXOneToManySource class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:32'!
descriptionLabel
	^ MAStringDescription new
		accessor: #label;
		priority: 100;
		label: 'Label';
		beXmlAttribute;
		yourself! !

!MXOneToManySource class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:34'!
descriptionTargets
	^(MAToManyRelationDescription new 
		accessor: #targets; 
		label: '1:m targets'; 
		priority: 600)
		reference: MXRelationTarget description;
		xmlElementName: 'elementTargets';
		beXmlExportable;
		yourself! !

!MXOneToManySource class methodsFor: 'testing' stamp: 'NorbertHartl 2/17/2010 06:54'!
isAbstract
	^ false! !

!MXOneToManySource class methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 02/18/2010 14:42'!
xmlElementName
	^ 'toManySource'! !

!MXOneToManySource methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 2/17/2010 08:59'!
inlineTargets
	^ inlineTargets! !

!MXOneToManySource methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 2/17/2010 08:59'!
inlineTargets: aCollection
	inlineTargets := aCollection! !

!MXOneToManySource methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:54'!
label
	^ label! !

!MXOneToManySource methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:54'!
label: aString
	label := aString! !

!MXOneToManySource methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:54'!
targets
	^ targets! !

!MXOneToManySource methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:54'!
targets: aCollection
	targets := aCollection! !

MXObject subclass: #MXOneToOneSource
	instanceVariableNames: 'label target inlineTarget'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXOneToOneSource class methodsFor: 'description' stamp: 'jvds 10/18/2010 16:19'!
descriptionInlineTarget
	^(MAToOneRelationDescription new 
		accessor: #inlineTarget; 
		label: '1:1 inline target'; 
		priority: 600)
		beXmlInlineElement;
		reference: MXRelationInlineTarget description;
		yourself! !

!MXOneToOneSource class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:34'!
descriptionLabel
	^ MAStringDescription new
		accessor: #label;
		priority: 100;
		label: 'Label';
		beXmlAttribute;
		yourself! !

!MXOneToOneSource class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:35'!
descriptionTarget
	^(MAToOneRelationDescription new 
		accessor: #target; 
		label: '1:1 target'; 
		priority: 600)
		xmlElementName: 'toOne';
		reference: MXRelationTarget description;
		beXmlExportable;
		yourself! !

!MXOneToOneSource class methodsFor: 'testing' stamp: 'NorbertHartl 2/17/2010 06:51'!
isAbstract
	^ false! !

!MXOneToOneSource class methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 02/18/2010 14:43'!
xmlElementName
	^ 'toOneSource'! !

!MXOneToOneSource methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 2/17/2010 08:35'!
inlineTarget
	^ inlineTarget! !

!MXOneToOneSource methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 2/17/2010 08:36'!
inlineTarget: anObject
	inlineTarget := anObject! !

!MXOneToOneSource methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:43'!
label
	^ label! !

!MXOneToOneSource methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:42'!
label: aString
	label := aString! !

!MXOneToOneSource methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:45'!
target
	^ target! !

!MXOneToOneSource methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:44'!
target: anObject
	target := anObject! !

MXObject subclass: #MXRelationEmbedSource
	instanceVariableNames: 'title target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXRelationEmbedSource class methodsFor: 'as yet unclassified' stamp: 'jvds 10/18/2010 18:39'!
descriptionTarget
	^MAToOneRelationDescription new 
		accessor: #target; 
		label: '1:1 embed target'; 
		priority: 600;
		xmlReadBlock: [ :node | (MXRelationEmbedTarget new) name: (node attributeAt: 'embedTargetName' )]
			writeBlock: [ :node :value | node attributeAt: 'embedTargetName' put: value name];
		reference: MXRelationEmbedTarget description;
		yourself! !

!MXRelationEmbedSource class methodsFor: 'description' stamp: 'jvds 8/31/2010 18:18'!
descriptionTitle
	^ MAStringDescription new
		accessor: #title;
		priority: 100;
		label: 'Title';
		beXmlAttribute;
		yourself! !

!MXRelationEmbedSource class methodsFor: 'testing' stamp: 'NorbertHartl 3/15/2010 18:53'!
isAbstract
	^ false! !

!MXRelationEmbedSource class methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 3/15/2010 18:56'!
xmlElementName
	^ 'relationEmbedSource'! !

!MXRelationEmbedSource methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 3/15/2010 19:01'!
target
	^ target! !

!MXRelationEmbedSource methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 3/15/2010 19:01'!
target: anObject
	target := anObject! !

!MXRelationEmbedSource methodsFor: 'accessing' stamp: 'NorbertHartl 3/15/2010 18:53'!
title
	^ title! !

!MXRelationEmbedSource methodsFor: 'accessing' stamp: 'NorbertHartl 3/15/2010 18:53'!
title: aString
	title := aString! !

MXObject subclass: #MXRelationEmbedTarget
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXRelationEmbedTarget class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:36'!
descriptionName
	^ MAStringDescription new
		accessor: #name;
		priority: 100;
		label: 'Name';
		beXmlAttribute;
		yourself! !

!MXRelationEmbedTarget class methodsFor: 'testing' stamp: 'NorbertHartl 3/15/2010 18:54'!
isAbstract
	^ false! !

!MXRelationEmbedTarget class methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 3/15/2010 18:56'!
xmlElementName
	^ 'relationEmbedTarget'! !

!MXRelationEmbedTarget methodsFor: 'accessing' stamp: 'NorbertHartl 3/15/2010 18:54'!
name
	^ name! !

!MXRelationEmbedTarget methodsFor: 'accessing' stamp: 'NorbertHartl 3/15/2010 18:54'!
name: aString
	name := aString! !

MXObject subclass: #MXRelationInlineTarget
	instanceVariableNames: 'title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXRelationInlineTarget class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:33'!
descriptionTitle
	^ MAStringDescription new
		accessor: #title;
		priority: 100;
		label: 'Title';
		beXmlAttribute;
		yourself! !

!MXRelationInlineTarget class methodsFor: 'testing' stamp: 'NorbertHartl 2/17/2010 13:36'!
isAbstract
	^ false! !

!MXRelationInlineTarget class methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 02/18/2010 14:43'!
xmlElementName
	^ 'inlineTarget'! !

!MXRelationInlineTarget methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 13:36'!
title
	^ title! !

!MXRelationInlineTarget methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 13:36'!
title: aString
	title := aString! !

MXObject subclass: #MXRelationTarget
	instanceVariableNames: 'title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXRelationTarget class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:34'!
descriptionTitle
	^ MAStringDescription new
		accessor: #title;
		priority: 100;
		label: 'Title';
		beXmlAttribute;
		yourself! !

!MXRelationTarget class methodsFor: 'testing' stamp: 'NorbertHartl 2/17/2010 06:51'!
isAbstract
	^ false! !

!MXRelationTarget class methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 02/18/2010 14:54'!
xmlElementName
	^ 'elementtarget'! !

!MXRelationTarget methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:42'!
title
	^ title! !

!MXRelationTarget methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 06:42'!
title: aString
	title := aString! !

MXObject subclass: #MXSimpleObject
	instanceVariableNames: 'title abstract token index date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXSimpleObject class methodsFor: 'description' stamp: 'jvds 10/10/2010 22:23'!
descriptionAbstract
	^ MAStringDescription new
		accessor: #abstract;
		priority: 150;
		label: 'Abstract';
		beXmlElement;
		yourself! !

!MXSimpleObject class methodsFor: 'description' stamp: 'jvds 10/18/2010 22:20'!
descriptionDate
	^ MADateDescription new
		accessor: #date;
		priority: 350;
		label: 'Date';
		beXmlElement;
		yourself! !

!MXSimpleObject class methodsFor: 'description' stamp: 'jvds 10/10/2010 22:23'!
descriptionIndex
	^ MANumberDescription new
		accessor: #index;
		priority: 200;
		label: 'Index';
		beXmlAttribute;	
		yourself! !

!MXSimpleObject class methodsFor: 'description' stamp: 'jvds 10/10/2010 22:22'!
descriptionTitle
	^ MAStringDescription new
		accessor: #title;
		priority: 100;
		label: 'Title';
		beXmlAttribute;
		yourself! !

!MXSimpleObject class methodsFor: 'description' stamp: 'jvds 10/14/2010 16:07'!
descriptionToken
	^ MAStringDescription new
		accessor: #token;
		priority: 180;
		label: 'Token';
		beXmlChildAttribute;
		xmlAttributeName: 'type';
		yourself! !

!MXSimpleObject class methodsFor: 'testing' stamp: 'NorbertHartl 2/17/2010 05:20'!
isAbstract
	^ false! !

!MXSimpleObject class methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 02/18/2010 14:44'!
xmlElementName
	^ 'simpleobject'! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:26'!
abstract
	^ abstract! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:26'!
abstract: aString
	abstract := aString! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:12'!
date
	^ date! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:12'!
date: aDate
	date := aDate! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:12'!
index
	^ index! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:12'!
index: aNumber
	index := aNumber! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:12'!
title
	^ title! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:11'!
title: aString
	title := aString! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:36'!
token
	^ token! !

!MXSimpleObject methodsFor: 'accessing' stamp: 'NorbertHartl 2/17/2010 05:37'!
token: aString
	token := aString! !

MXObject subclass: #MXTypesObject
	instanceVariableNames: 'string number date boolean magnitudeClass colors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXTypesObject class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:41'!
descriptionBoolean
	^ MABooleanDescription new
		accessor: #boolean;
		priority: 120;
		label: 'Boolean';
		beXmlAttribute;
		yourself! !

!MXTypesObject class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:22'!
descriptionColors

	^ MAMultipleOptionDescription new
		accessor: #colors;
		options: #( red yellow blue green);
		accessor: #colors;
		beXmlExportable;
		yourself
! !

!MXTypesObject class methodsFor: 'description' stamp: 'jvds 10/14/2010 15:02'!
descriptionDate
	^ MADateDescription new
		accessor: #date;
		priority: 110;
		label: 'Date';
		beXmlAttribute;
		yourself! !

!MXTypesObject class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:59'!
descriptionMagnitudeClass

	^ MASingleOptionDescription new
		options: Magnitude allSubclasses;
		reference: MAClassDescription new;
		accessor: #magnitudeClass;
		beXmlAttribute;
		yourself
! !

!MXTypesObject class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:57'!
descriptionNumber
	^ MANumberDescription new
		accessor: #number;
		priority: 105;
		label: 'Number';
		beXmlAttribute;	
		yourself! !

!MXTypesObject class methodsFor: 'description' stamp: 'jvds 10/14/2010 14:58'!
descriptionString
	^ MAStringDescription new
		accessor: #string;
		priority: 100;
		label: 'String';
		beXmlAttribute;
		yourself! !

!MXTypesObject class methodsFor: 'testing' stamp: 'NorbertHartl 3/16/2010 15:14'!
isAbstract
	^ false! !

!MXTypesObject class methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 3/16/2010 15:23'!
xmlElementName
	^ 'typesobject'! !

!MXTypesObject methodsFor: 'accessing' stamp: 'jvds 8/5/2010 00:03'!
boolean
	^ boolean! !

!MXTypesObject methodsFor: 'accessing' stamp: 'jvds 8/5/2010 00:03'!
boolean: anObject
	boolean := anObject! !

!MXTypesObject methodsFor: 'accessing' stamp: 'jvds 8/31/2010 18:34'!
colors
	^ colors! !

!MXTypesObject methodsFor: 'accessing' stamp: 'jvds 8/31/2010 18:34'!
colors: anObject
	colors := anObject! !

!MXTypesObject methodsFor: 'accessing' stamp: 'NorbertHartl 3/16/2010 15:14'!
date
	^ date! !

!MXTypesObject methodsFor: 'accessing' stamp: 'NorbertHartl 3/16/2010 15:14'!
date: aDate
	date := aDate! !

!MXTypesObject methodsFor: 'accessing' stamp: 'jvds 8/8/2010 15:28'!
magnitudeClass
	^ magnitudeClass! !

!MXTypesObject methodsFor: 'accessing' stamp: 'jvds 8/8/2010 15:28'!
magnitudeClass: anObject
	magnitudeClass := anObject! !

!MXTypesObject methodsFor: 'accessing' stamp: 'NorbertHartl 3/16/2010 15:17'!
number
	^ number! !

!MXTypesObject methodsFor: 'accessing' stamp: 'NorbertHartl 3/16/2010 15:17'!
number: aNumber
	number := aNumber! !

!MXTypesObject methodsFor: 'accessing' stamp: 'NorbertHartl 3/16/2010 15:16'!
string
	^ string! !

!MXTypesObject methodsFor: 'accessing' stamp: 'NorbertHartl 3/16/2010 15:16'!
string: aString
	string := aString! !

Object subclass: #MXPluggableXmlAccessor
	instanceVariableNames: 'readBlock writeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXPluggableXmlAccessor commentStamp: 'jvds 10/18/2010 21:32' prior: 0!
MXPluggableXmlAccessor can be used as an alternative to the MXXmlWriter and MXXmlReader classes for customized mapping of an object to xml.

Instance Variables:
	readBlock	<Block>
	writeBlock	<Block>!

!MXPluggableXmlAccessor methodsFor: 'model' stamp: 'jvds 10/10/2010 15:26'!
read: aNode description: aDescription
	^self readBlock value: aNode! !

!MXPluggableXmlAccessor methodsFor: 'accessing' stamp: 'jvds 10/10/2010 15:22'!
readBlock
	^ readBlock! !

!MXPluggableXmlAccessor methodsFor: 'accessing' stamp: 'jvds 10/10/2010 15:22'!
readBlock: anObject
	readBlock := anObject! !

!MXPluggableXmlAccessor methodsFor: 'model' stamp: 'jvds 10/10/2010 15:42'!
write: anObject description: aDescription to: aNode
	self writeBlock 
		value: aNode value: (anObject readUsing: aDescription)! !

!MXPluggableXmlAccessor methodsFor: 'accessing' stamp: 'jvds 10/10/2010 15:22'!
writeBlock
	^ writeBlock! !

!MXPluggableXmlAccessor methodsFor: 'accessing' stamp: 'jvds 10/10/2010 15:22'!
writeBlock: anObject
	writeBlock := anObject! !

Object subclass: #MXTestAddress
	instanceVariableNames: 'postalCode city'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXTestAddress class methodsFor: 'as yet unclassified' stamp: 'jvds 10/7/2010 22:10'!
descriptionCity

	^MAStringDescription new
		accessor: #city;
		label: 'City';
		beXmlElement;
		priority: 20;
		yourself! !

!MXTestAddress class methodsFor: 'as yet unclassified' stamp: 'jvds 10/22/2010 09:25'!
descriptionPostalCode

	^MAStringDescription new
		accessor: #postalCode;
		label: 'Postal Code';
		beXmlAttribute;
		priority: 15;
		yourself! !

!MXTestAddress methodsFor: 'accessing' stamp: 'jvds 10/5/2010 22:42'!
city
	^ city! !

!MXTestAddress methodsFor: 'accessing' stamp: 'jvds 10/5/2010 22:42'!
city: anObject
	city := anObject! !

!MXTestAddress methodsFor: 'accessing' stamp: 'jvds 10/22/2010 09:24'!
postalCode
	^ postalCode! !

!MXTestAddress methodsFor: 'accessing' stamp: 'jvds 10/22/2010 09:24'!
postalCode: anObject
	postalCode := anObject! !

MXTestAddress subclass: #MXTestPOBoxAddress
	instanceVariableNames: 'pobox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXTestPOBoxAddress class methodsFor: 'as yet unclassified' stamp: 'jvds 10/22/2010 09:27'!
descriptionStreet

	^MANumberDescription new
		accessor: #pobox;
		label: 'PO Box';
		beInteger;
		beXmlElement;
		priority: 10;
		yourself! !

!MXTestPOBoxAddress methodsFor: 'accessing' stamp: 'jvds 10/22/2010 09:23'!
pobox
	^ pobox! !

!MXTestPOBoxAddress methodsFor: 'accessing' stamp: 'jvds 10/22/2010 09:23'!
pobox: anObject
	pobox := anObject! !

MXTestAddress subclass: #MXTestStreetAddress
	instanceVariableNames: 'street'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXTestStreetAddress class methodsFor: 'as yet unclassified' stamp: 'jvds 10/22/2010 09:25'!
descriptionStreet

	^MAStringDescription new
		accessor: #street;
		label: 'Street';
		beXmlElement;
		priority: 10;
		yourself! !

!MXTestStreetAddress methodsFor: 'accessing' stamp: 'jvds 10/22/2010 09:23'!
street
	^ street! !

!MXTestStreetAddress methodsFor: 'accessing' stamp: 'jvds 10/22/2010 09:23'!
street: anObject
	street := anObject! !

Object subclass: #MXTestPerson
	instanceVariableNames: 'name address favouriteHobby otherHobbies otherAddresses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding-Test'!

!MXTestPerson class methodsFor: 'as yet unclassified' stamp: 'jvds 10/7/2010 18:43'!
allHobbies

	^#( 'Tennis' 'Golf' 'Football' 'Chess' 'Smalltalk programming' )! !

!MXTestPerson class methodsFor: 'as yet unclassified' stamp: 'jvds 10/9/2010 23:54'!
descriptionAddress

	^MAToOneRelationDescription new
		accessor: #address;
		label: 'Address';
		classes: (Array with: MXTestAddress);
		beXmlElement;
		priority: 20;
		yourself! !

!MXTestPerson class methodsFor: 'as yet unclassified' stamp: 'jvds 10/7/2010 22:09'!
descriptionFavouriteHobby

	^MASingleOptionDescription new
		accessor: #favouriteHobby;
		label: 'Favourite hobby';
		options: self allHobbies;
		beXmlElement;
		priority: 30;
		yourself! !

!MXTestPerson class methodsFor: 'as yet unclassified' stamp: 'jvds 10/7/2010 22:08'!
descriptionName

	^MAStringDescription new
		accessor: #name;
		label: 'Name';
		beXmlAttribute;
		priority: 10;
		yourself! !

!MXTestPerson class methodsFor: 'as yet unclassified' stamp: 'jvds 10/9/2010 23:54'!
descriptionOtherAddresses

	^MAToManyRelationDescription new
		accessor: #otherAddresses;
		label: 'Other addresses';
		classes: (Array with: MXTestAddress);
		beXmlElement;
		priority: 40;
		yourself! !

!MXTestPerson class methodsFor: 'as yet unclassified' stamp: 'jvds 10/7/2010 22:10'!
descriptionOtherHobbies

	^MAMultipleOptionDescription new
		accessor: #otherHobbies;
		label: 'Other hobbies';
		options: self allHobbies;
		beXmlElement;
		priority: 50;
		yourself! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/5/2010 21:21'!
address
	^ address! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/5/2010 21:21'!
address: anObject
	address := anObject! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/7/2010 18:32'!
favouriteHobby
	^ favouriteHobby! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/7/2010 18:32'!
favouriteHobby: anObject
	favouriteHobby := anObject! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/5/2010 21:21'!
name
	^ name! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/5/2010 21:21'!
name: anObject
	name := anObject! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/7/2010 18:32'!
otherAddresses
	^ otherAddresses! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/7/2010 18:32'!
otherAddresses: anObject
	otherAddresses := anObject! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/7/2010 18:32'!
otherHobbies
	^ otherHobbies! !

!MXTestPerson methodsFor: 'accessing' stamp: 'jvds 10/7/2010 18:32'!
otherHobbies: anObject
	otherHobbies := anObject! !

Object subclass: #MXXmlTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlTarget commentStamp: 'jvds 10/18/2010 21:37' prior: 0!
MXXmlTarget is the abstract root of a utitity class hierarchy used by MXXmlReader and MXXmlWriter to access the xml node tree structure.!

MXXmlTarget subclass: #MXXmlAttributeTarget
	instanceVariableNames: 'attributeName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlAttributeTarget commentStamp: 'jvds 10/18/2010 21:39' prior: 0!
MXXmlAttributeTarget stores and reads text from xml attributes. For example:

	<container attributeName="value"></container>

Instance Variables:
	attributeName	<String>!

!MXXmlAttributeTarget methodsFor: 'writing' stamp: 'jvds 10/10/2010 21:55'!
addText: aString to: aContainerNode
	"Store ==aString== as an attribute in the element ==aContainerNode==."
	

	^aContainerNode
		attributeAt: self attributeName put: aString;
		yourself! !

!MXXmlAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/4/2010 22:33'!
attributeName
	^ attributeName! !

!MXXmlAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/4/2010 22:34'!
attributeName: aString
	attributeName := aString! !

!MXXmlAttributeTarget methodsFor: 'reading' stamp: 'jvds 10/6/2010 22:55'!
getTextFrom: anElement

	^anElement attributeAt: self attributeName ifAbsent: [ nil ]! !

MXXmlAttributeTarget subclass: #MXXmlNestedAttributeTarget
	instanceVariableNames: 'elementName useExisting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlNestedAttributeTarget commentStamp: 'jvds 10/18/2010 21:43' prior: 0!
MXXmlNestedAttributeTarget stores and reads text from xml attributes. For example:

	<container><elementName attributeName="value" /></container>

Instance Variables:
	elementName	<String>
	useExisting	<Boolean> - If the container elements already contains an element with the correct name and useExisting is true, 
		the attribute will be added to the existing element. Otherwise a new element is created.!

!MXXmlNestedAttributeTarget methodsFor: 'writing' stamp: 'NorbertHartl 12/21/2010 14:32'!
addText: aString to: aContainerNode
	"Store ==aString== as an attribute in the element ==aContainerNode==."
	
	| childElement |

	childElement := self useExisting 
		ifTrue: [ aContainerNode elementAt: self elementName ]
		ifFalse: [ nil ].
	childElement isNil
		ifTrue: [
			childElement := XMLElement named: self elementName.
			aContainerNode 
				addNode: childElement ].
	^childElement
		attributeAt: self attributeName put: aString;
		yourself
! !

!MXXmlNestedAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:06'!
elementName
	^ elementName! !

!MXXmlNestedAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:06'!
elementName: anObject
	elementName := anObject! !

!MXXmlNestedAttributeTarget methodsFor: 'reading' stamp: 'jvds 10/10/2010 22:08'!
getTextFrom: anElement

	| childElement |
	childElement := anElement elementAt: self elementName.
	^childElement isNil
		ifTrue: [ nil ]
		ifFalse: [ childElement attributeAt: self attributeName ]! !

!MXXmlNestedAttributeTarget methodsFor: 'initialize-release' stamp: 'jvds 10/10/2010 22:12'!
initialize

	super initialize.
	useExisting := false! !

!MXXmlNestedAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:38'!
useExisting
	^useExisting! !

!MXXmlNestedAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:38'!
useExisting: aBoolean
	useExisting := aBoolean! !

MXXmlTarget subclass: #MXXmlElementTarget
	instanceVariableNames: 'inlined'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlElementTarget commentStamp: 'jvds 10/18/2010 21:45' prior: 0!
MXXmlElementTarget is an abstract class with support for storing and reading text as the contents of an element. And for adding 
and reading child elements.!

MXXmlElementTarget subclass: #MXXmlClassNameElementTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlClassNameElementTarget commentStamp: 'jvds 10/18/2010 21:48' prior: 0!
MXXmlClassNameElementTarget uses the class name of the object as the element name. For example:

	<MXTestPerson>........</MXTestPerson>!

!MXXmlClassNameElementTarget methodsFor: 'internal' stamp: 'jvds 10/22/2010 10:02'!
elementNameFor: anObject

	^anObject class name greaseString! !

!MXXmlClassNameElementTarget methodsFor: 'reading' stamp: 'jvds 10/19/2010 23:25'!
getInstanceFrom: anElement description: aDescription

	| class |
	class := Smalltalk environment 
		at: anElement name asSymbol
		ifAbsent: [ ^nil ].
	^class new! !

!MXXmlElementTarget methodsFor: 'writing' stamp: 'NorbertHartl 12/21/2010 15:12'!
addNodeForInstance: anObject to: aContainerNode
	"Create and answer a new element node that represents ==anObject==."
	
	self isInlined ifTrue: [ ^ aContainerNode ].
	^ self addNodeNamed: (self elementNameFor: anObject) toNode: aContainerNode
! !

!MXXmlElementTarget methodsFor: 'writing' stamp: 'NorbertHartl 12/21/2010 15:12'!
addNodeNamed: aString toNode: aContainerNode
	"Create and answer a new element node. Add it to ==aContainerNode=="
	
	| node |
	node := XMLElement named: aString.
	aContainerNode addNode: node.
	^ node! !

!MXXmlElementTarget methodsFor: 'writing' stamp: 'NorbertHartl 12/21/2010 15:12'!
addNodeTo: aContainerNode
	"Create and answer a new element node. Add it to ==aContainerNode=="
	
	self isInlined ifTrue: [ ^ aContainerNode ].
	^ self addNodeNamed: self elementName toNode: aContainerNode 
! !

!MXXmlElementTarget methodsFor: 'writing' stamp: 'jvds 10/22/2010 10:59'!
addText: aString to: aContainerNode
	"Create a new element with ==aString== as the contents. Add the new element to ==aContainerNode== if present.
	Answer the newly created element."

	| node |
	
	node := self addNodeTo: aContainerNode.
	
	self useCDATASection 
		ifTrue: [ node addNode: (XMLCData string: aString) ]
		ifFalse: [ node addContent: aString ].
	^node! !

!MXXmlElementTarget methodsFor: 'accessing' stamp: 'NorbertHartl 11/13/2010 17:26'!
beInlined
	inlined := true! !

!MXXmlElementTarget methodsFor: 'accessing' stamp: 'jvds 10/5/2010 23:20'!
elementName
	^self subclassResponsibility! !

!MXXmlElementTarget methodsFor: 'internal' stamp: 'jvds 10/10/2010 22:00'!
elementNameFor: anObject

	^self elementName! !

!MXXmlElementTarget methodsFor: 'reading' stamp: 'jvds 10/19/2010 23:24'!
getInstanceFrom: anElement description: aDescription
	"Answer an instance of the correct class for ==anElement==."

	aDescription kind = Object
		ifTrue: [ 
			"assume a subclass of MXObject"
			MXObject withAllSubclassesDo: [ :class |
				(class isAbstract not and: [ class description xmlElementName = anElement name ])
					ifTrue: [ ^class new ]]]
		ifFalse: [ 
			aDescription kind withAllSubclassesDo: [ :class |
				class description xmlElementName = anElement name
					ifTrue: [ ^class new ]]].

	^nil! !

!MXXmlElementTarget methodsFor: 'reading' stamp: 'NorbertHartl 11/13/2010 17:28'!
getNodeFrom: aContainerNode
	self isInlined ifTrue: [
		^ aContainerNode ].
	^aContainerNode elementAt: self elementName! !

!MXXmlElementTarget methodsFor: 'reading' stamp: 'NorbertHartl 11/29/2010 21:51'!
getTextFrom: aContainerNode

	^(self getNodeFrom: aContainerNode)
		ifNotNilDo: [ :elem | elem contentString ]! !

!MXXmlElementTarget methodsFor: 'initialize-release' stamp: 'NorbertHartl 11/13/2010 18:47'!
initialize
	super initialize.
	inlined := false! !

!MXXmlElementTarget methodsFor: 'testing' stamp: 'NorbertHartl 11/13/2010 17:27'!
isInlined
	^ inlined ifNil: [
		inlined := false]! !

!MXXmlElementTarget methodsFor: 'testing' stamp: 'jvds 10/22/2010 10:55'!
useCDATASection

	^false! !

MXXmlElementTarget subclass: #MXXmlNamedElementTarget
	instanceVariableNames: 'elementName useCDATASection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlNamedElementTarget commentStamp: 'jvds 10/18/2010 21:49' prior: 0!
MXXmlNamedElementTarget stores text and child elements in an element with a fixed name.

	<elementName>........</elementName>

Instance Variables:
	elementName	<String>!

MXXmlNamedElementTarget subclass: #MXXmlClassNameAttributeTarget
	instanceVariableNames: 'classAttributeName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlClassNameAttributeTarget commentStamp: 'jvds 10/18/2010 21:50' prior: 0!
MXXmlNamedElementTarget stores text and child elements in an element with a fixed name. The class name of the stored object is saved in an attribute.

	<elementName type="MXTestPerson">........</elementName>

Instance Variables:
	classAttributeName <String>!

!MXXmlClassNameAttributeTarget methodsFor: 'writing' stamp: 'jvds 10/10/2010 22:52'!
addNodeForInstance: anObject to: aContainerNode
	"Create and answer a new element node that represents ==anObject==."

	^(super addNodeForInstance: anObject to: aContainerNode)
		attributeAt: self classAttributeName put: anObject class name;
		yourself! !

!MXXmlClassNameAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:48'!
classAttributeName
	^ classAttributeName! !

!MXXmlClassNameAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:48'!
classAttributeName: anObject
	classAttributeName := anObject! !

!MXXmlClassNameAttributeTarget methodsFor: 'reading' stamp: 'jvds 10/19/2010 23:24'!
getInstanceFrom: anElement description: aDescription

	| name class |
	name := anElement attributeAt: self classAttributeName.
	class := Smalltalk environment 
		at: name asSymbol
		ifAbsent: [ ^nil ].
	^class new! !

!MXXmlClassNameAttributeTarget methodsFor: 'initialize-release' stamp: 'NorbertHartl 11/15/2010 00:14'!
initialize

	super initialize.
	classAttributeName := 'type'
! !

!MXXmlNamedElementTarget methodsFor: 'accessing' stamp: 'jvds 10/5/2010 23:21'!
elementName

	^elementName ! !

!MXXmlNamedElementTarget methodsFor: 'accessing' stamp: 'jvds 10/5/2010 23:21'!
elementName: aString

	elementName := aString! !

!MXXmlNamedElementTarget methodsFor: 'initialize-release' stamp: 'jvds 10/22/2010 10:52'!
initialize

	super initialize.
	useCDATASection := false! !

!MXXmlNamedElementTarget methodsFor: 'accessing' stamp: 'jvds 10/22/2010 10:52'!
setUseCDATASection

	useCDATASection := true! !

!MXXmlNamedElementTarget methodsFor: 'accessing' stamp: 'jvds 10/22/2010 10:55'!
useCDATASection

	^useCDATASection! !

!MXXmlTarget methodsFor: 'writing' stamp: 'jvds 10/4/2010 22:05'!
addText: aString to: aContainerNode

	^aContainerNode
		addContent: aString;
		yourself! !

!MXXmlTarget methodsFor: 'writing' stamp: 'jvds 10/14/2010 14:53'!
addUndefinedTo: aContainerNode
	"Add a undefined (nil) value to aContainerNode. The default implementation does nothing"

	^aContainerNode! !

!MXXmlTarget methodsFor: 'reading' stamp: 'jvds 10/10/2010 21:58'!
getTextFrom: anElement

	^anElement contentString! !
